"""
Set the default vault alias to use.
"""

from sys import stderr

from rich import print
from typer import Argument

from vau.config import Config


def main(
    alias: str = Argument(..., help="The alias to set as default"),
    *,
    config: Config,
) -> None:
    """Set the default vault alias to use."""

    if alias not in config.aliases:
        print(f"[red]Unknown vault alias '{alias}'[/red]", file=stderr)
        exit(1)
    config.default_alias = alias
    config.to_file()



"""
A helper module to reflect on runtime objects, their methods and attributes. Allows you to build a proxy object that
behaves like the original object, but with some modifications.
"""

from __future__ import annotations

import functools
import inspect
from typing import Any, Callable, Iterable, TypeVar, cast

from wrapt import ObjectProxy  # type: ignore[import]

T = TypeVar("T")


class Method:
    def __init__(self, refl: Reflection, name: str) -> None:
        self.name = name
        self._refl = refl
        self._signature: inspect.Signature | None = None
        self._wrapper: Callable[..., Any] | None = None
        self.kwargs: dict[str, Any] = {}

    @property
    def original(self) -> Callable[..., Any]:
        return getattr(self._refl._obj, self.name)  # type: ignore[no-any-return]

    @property
    def signature(self) -> inspect.Signature:
        if self._signature is None:
            self._signature = inspect.signature(self.original)
        return self._signature

    def wrapper(self, wrapper: Callable[..., Any]) -> Callable[..., Any]:
        self._wrapper = functools.wraps(self.original)(wrapper)
        return wrapper

    def build(self) -> Callable[..., Any]:
        original = self.original
        if not self.kwargs and self._wrapper is not None:
            return original

        if not self.kwargs:
            assert self._wrapper is not None
            return self._wrapper

        @functools.wraps(original)
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            if self._wrapper is not None:
                return self._wrapper(*args, **{**self.kwargs, **kwargs})
            else:
                return original(*args, **{**self.kwargs, **kwargs})

        return wrapper


class Reflection:
    def __init__(self, obj: Any) -> None:
        self._obj = obj
        self._methods: dict[str, Method] = {}

    def methods(self) -> Iterable[Method]:
        # NOTE(@NiklasRosenstein): inspect.signature() won't work with e.g. __subclasshook__ and __init_subclass__.
        return (
            self.method(name)
            for name in dir(self._obj)
            if callable(getattr(self._obj, name))
            if not name.startswith("__")
        )

    def method(self, name: str) -> Method:
        if name not in self._methods:
            self._methods[name] = Method(self, name)
        return self._methods[name]

    def into(self, cls: type[T]) -> T:
        """Creates an #ObjectProxy that behaves like the original object, but with some modifications."""

        # NOTE(@NiklasRosenstein): Dirty hack because we can't use object.__setattr__ on an #ObjectProxy
        #       without also setting the WRAPT_DISABLE_EXTENSIONS environment variable. See this issue:
        #       https://github.com/GrahamDumpleton/wrapt/issues/235
        attrs = {}
        for method in self._methods.values():
            attrs[method.name] = staticmethod(method.build())

        proxy_cls = type("Proxy", (ObjectProxy,), attrs)
        proxy = proxy_cls(self._obj)
        return cast(T, proxy)



from __future__ import annotations

import json
from pathlib import Path
from sys import stderr
from typing import Optional

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Context, Option

from .._format import SecretIdFormat
from .._utils import comma_flat_split

# NOTE(@NiklasRosenstein): We're using `Optional[str]` instead of `str | None` here because of a Black formatting
#       bug described in https://github.com/psf/black/issues/3649.

# NOTE(@NiklasRosenstein): It appears that `num_uses` and `ttl` is not currently supported by hvac.


def main(
    client: AppRole,
    ctx: Context,
    secret_id: Optional[str] = Option(
        None,
        help="A custom secret ID to use. If not specified, one will be generated.",
        metavar="SECRET_ID",
    ),
    metadata: Optional[str] = Option(
        None, help="JSON-formatted metadata to associate with the secret ID", metavar="JSON"
    ),
    metadata_file: Optional[Path] = Option(
        None,
        help="A file containing JSON-formatted metadata to associate with the secret ID",
    ),
    cidr: list[str] | None = Option(None, help="The CIDRs to bind the secret ID to", metavar="CIDR"),
    token_bound_cidrs: list[str] | None = Option(None, help="The CIDRs to bind the token to", metavar="CIDR"),
    format: SecretIdFormat = Option(SecretIdFormat.json, "-o", "--format", help="Output format"),
) -> None:
    """
    Create a new secret ID for an AppRole.
    """

    role_name = ctx.ensure_object(str)

    if metadata_file is not None:
        if metadata is not None:
            print("Cannot specify both --metadata and --metadata-file", file=stderr)
            exit(1)
        metadata_json = json.loads(metadata_file.read_text())
    else:
        metadata_json = json.loads(metadata or "{}")

    if secret_id is None:
        secretid = client.generate_secret_id(
            role_name,
            metadata=metadata_json,
            cidr_list=comma_flat_split(cidr or []),
            token_bound_cidrs=comma_flat_split(token_bound_cidrs or []),
        )["data"]
    else:
        secretid = client.create_custom_secret_id(
            role_name=role_name,
            secret_id=secret_id,
            metadata=metadata_json,
            cidr_list=comma_flat_split(cidr or []),
            token_bound_cidrs=comma_flat_split(token_bound_cidrs or []),
        )["data"]

    print(format.formatter.format_secretid(secretid))



"""
Remove an AppRole from vault.
"""

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Option


def main(
    client: AppRole,
    role_name: str = Option(..., help="The name of the approle", metavar="ROLE_NAME"),
) -> None:
    """
    Remove a vault AppRole.

    Note that removing an AppRole that does not exist will not result in an error.
    """

    client.delete_role(role_name)
    print(f"Removed AppRole [bold]{role_name}[/bold].")



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path

import databind.json


@dataclass
class VaultAlias:
    addr: str
    token: str | None
    namespace: str | None


@dataclass
class Config:
    aliases: dict[str, VaultAlias]
    default_alias: str | None

    def __post_init__(self) -> None:
        self.path: Path | None = None

    @staticmethod
    def from_file(file: Path) -> Config:
        if file.exists():
            with file.open("r") as fp:
                raw_data = json.load(fp)
            config = databind.json.load(raw_data, Config)
        else:
            config = Config(aliases={}, default_alias=None)
        config.path = file
        return config

    def to_file(self, file: Path | None = None) -> None:
        if file is None:
            if self.path is None:
                raise ValueError("no path specified")
            file = self.path
        file.parent.mkdir(parents=True, exist_ok=True)
        with file.open("w") as fp:
            json.dump(databind.json.dump(self, Config), fp, indent=2)



from __future__ import annotations

from rich.console import Console
from rich.table import Table
from typer import Option

from vau.config import Config


def main(
    show_tokens: bool = Option(False, help="Show tokens in the table"),
    *,
    config: Config,
) -> None:
    """
    List all aliases in a rich table.
    """

    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Alias", style="dim", width=12)
    table.add_column("Vault Address", style="dim")
    table.add_column("Token", style="dim")
    table.add_column("Namespace", style="dim")

    for alias, details in config.aliases.items():
        table.add_row(
            f"[bold green]{alias}[/bold green]" if alias == config.default_alias else alias,
            details.addr,
            details.token if show_tokens else "********" if details.token else "",
            details.namespace or "-",
        )

    console = Console()
    console.print(table)



"""
Manage AppRoles.

AppRoles are a way to authenticate against Vault. They are similar to AWS IAM roles.
"""

from __future__ import annotations

from typing import Any

from hvac import Client  # type: ignore[import]
from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from hvac.constants.approle import DEFAULT_MOUNT_POINT  # type: ignore[import]
from typer import Option
from typer_builder import Dependencies

from vau.reflection import Reflection

# NOTE(@NiklasRosenstein): Details on the AppRole API can be found here:
#       https://developer.hashicorp.com/vault/api-docs/auth/approle


def callback(
    client: Client,
    dependencies: Dependencies = Dependencies.Provides(AppRole),
    mount_point: str = Option(DEFAULT_MOUNT_POINT, help="Approle mount point", metavar="MOUNT_POINT"),
) -> None:
    # Inject the mount_point option as default value into all methods of the approle object that accept it.
    refl = Reflection(client.auth.approle)
    for method in refl.methods():
        if "mount_point" in method.signature.parameters:
            method.kwargs["mount_point"] = mount_point

    # Ensure that read_role() returns the role id as well.
    method = refl.method("read_role")

    @method.wrapper
    def _read_role(role_name: str, **kwargs: Any) -> dict[str, str]:
        role = method.original(role_name, **kwargs)
        role_id = client.auth.approle.read_role_id(role_name, **kwargs)["data"]["role_id"]
        role["data"]["role_name"] = role_name
        role["data"]["role_id"] = role_id
        return role  # type: ignore[no-any-return]

    dependencies.set(AppRole, refl.into(AppRole))


def list_roles(client: AppRole) -> list[dict[str, Any]]:
    """
    List all vault app roles and their role ids.
    """

    role_names: list[str] = client.list_roles()["data"]["keys"]
    return [
        {
            **client.read_role(role_name)["data"],
            "role_name": role_name,
            "role_id": client.read_role_id(role_name)["data"]["role_id"],
        }
        for role_name in role_names
    ]



"""
Manage AppRole Role IDs.

Role IDs are used to authenticate against an AppRole. Role IDs are not secret and can be distributed freely.
"""

from typer import Context, Option


def callback(
    ctx: Context,
    role_name: str = Option(..., help="The name of the approle", metavar="ROLE_NAME"),
) -> None:
    ctx.obj = role_name



from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Option

from ._format import ApproleFormat


def main(
    client: AppRole,
    role_name: str = Option(..., help="The name of the approle", metavar="ROLE_NAME"),
    format: ApproleFormat = Option(ApproleFormat.json, "-o", "--format", help="Output format"),
) -> None:
    """Get a vault approle."""

    approle = client.read_role(role_name)["data"]
    print(format.formatter.format_approle(approle))



from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from typer import Context


def main(client: AppRole, ctx: Context) -> None:
    role_name = ctx.ensure_object(str)
    role_id = client.read_role_id(role_name)["data"]["role_id"]
    print(role_id)



from sys import stderr

from rich import print
from typer import Argument

from vau.config import Config


def main(
    alias: str = Argument(..., help="The alias to delete"),
    *,
    config: Config,
) -> None:
    """Delete a locally configured Vault alias."""

    if alias not in config.aliases:
        print(f"[red]Unknown alias '{alias}'[/red]", file=stderr)
        exit(1)

    del config.aliases[alias]
    if config.default_alias == alias:
        config.default_alias = None
    config.to_file()



"""
Manage AppRole Secret IDs and Secret ID accessors.

Secret IDs are used to authenticate against an AppRole. Secret ID accessors are used to revoke Secret IDs.
"""

from typer import Context, Option


def callback(
    ctx: Context,
    role_name: str = Option(..., help="The name of the approle", metavar="ROLE_NAME"),
) -> None:
    ctx.obj = role_name



from __future__ import annotations

from itertools import chain
from typing import Iterable


def comma_flat_split(values: Iterable[str]) -> list[str]:
    """Split a comma-separated list of values into a flat list of values."""
    return list(chain.from_iterable(value.split(",") for value in values))



from vau.reflection import Reflection


def test_reflection_into_behaves_different_than_wrapped_object() -> None:
    class A:
        def foo(self, a: int = 1) -> int:
            return a

    a = A()
    refl = Reflection(a)
    refl.method("foo").kwargs["a"] = 2
    b = refl.into(A)

    assert a.foo() == 1
    assert b.foo() == 2



from __future__ import annotations

from typer import Option

from vau.config import Config, VaultAlias


def main(
    alias: str = Option(..., help="The alias to create or update"),
    vault_addr: str = Option(..., help="The address of the vault server"),
    token: str | None = Option(None, help="The vault token"),
    namespace: str | None = Option(None, help="The vault namespace"),
    use: bool = Option(True, help="Use the alias after creating it"),
    *,
    config: Config,
) -> None:
    """Create or update a locally configured Vault alias."""

    config.aliases[alias] = VaultAlias(addr=vault_addr, token=token, namespace=namespace)
    if use:
        config.default_alias = alias
    config.to_file()



from __future__ import annotations

import json
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from io import StringIO
from typing import Any

from rich.console import Console
from rich.table import Table


def _fmt_value(value: Any) -> str:
    if isinstance(value, bool):
        return str(value).lower()
    if value is None:
        return "null"
    return repr(value)


###
# ApproleFormatter


class ApproleFormatter(ABC):
    """
    Abstract base class for formatting approles.
    """

    @abstractmethod
    def format_approle(self, approle: dict[str, Any]) -> str:
        ...


@dataclass
class ApproleJsonFormatter(ApproleFormatter):
    indent: int = 2
    sort_keys: bool = True

    def format_approle(self, approle: dict[str, Any]) -> str:
        return json.dumps(approle, indent=self.indent, sort_keys=self.sort_keys)


class ApproleFormat(str, Enum):
    json = "json"

    @property
    def formatter(self) -> ApproleFormatter:
        if self == ApproleFormat.json:
            return ApproleJsonFormatter()
        else:
            raise ValueError(f"Unknown formatter: {self}")


###
# ApproleListFormatter


class ApproleListFormatter(ABC):
    @abstractmethod
    def format_approles(self, approles: list[dict[str, Any]]) -> str:
        ...


@dataclass
class ApproleListJsonFormatter(ApproleListFormatter):
    indent: int = 2
    sort_keys: bool = True

    def format_approles(self, approles: list[dict[str, Any]]) -> str:
        return json.dumps(approles, indent=self.indent, sort_keys=self.sort_keys)


class ApproleListTableFormatter(ApproleListFormatter):
    def format_approles(self, approles: list[dict[str, Any]]) -> str:
        table = Table()
        table.add_column("Role Name")
        table.add_column("Role ID")
        table.add_column("Token Policies")
        table.add_column("Token TTL")
        table.add_column("Token Type")

        for approle in approles:
            table.add_row(
                approle["role_name"],
                approle["role_id"],
                _fmt_value(approle["token_policies"]),
                _fmt_value(approle["token_ttl"]),
                approle["token_type"],
            )

        buffer = StringIO()
        Console(file=buffer).print(table)
        return buffer.getvalue().rstrip()


class ApproleListFormat(str, Enum):
    table = "table"
    json = "json"

    @property
    def formatter(self) -> ApproleListFormatter:
        if self == ApproleListFormat.table:
            return ApproleListTableFormatter()
        elif self == ApproleListFormat.json:
            return ApproleListJsonFormatter()
        else:
            raise ValueError(f"Unknown formatter: {self}")


###
# SecretIdFormatter


class SecretIdFormatter(ABC):
    @abstractmethod
    def format_secretid(self, secretid: dict[str, Any]) -> str:
        ...


class SecretIdJsonFormatter(SecretIdFormatter):
    indent: int = 2
    sort_keys: bool = True

    def format_secretid(self, secretid: dict[str, Any]) -> str:
        return json.dumps(secretid, indent=self.indent, sort_keys=self.sort_keys)


class SecretIdFormat(str, Enum):
    json = "json"

    @property
    def formatter(self) -> SecretIdFormatter:
        if self == SecretIdFormat.json:
            return SecretIdJsonFormatter()
        else:
            raise ValueError(f"Unknown formatter: {self}")


###
# SecretIdAccessorListFormatter


class SecretIdAccessorListFormatter(ABC):
    @abstractmethod
    def format_secretidaccessors(self, secretidaccessors: list[dict[str, Any]]) -> str:
        ...


@dataclass
class SecretIdAccessorListJsonFormatter(SecretIdAccessorListFormatter):
    indent: int = 2
    sort_keys: bool = True

    def format_secretidaccessors(self, secretidaccessors: list[dict[str, Any]]) -> str:
        return json.dumps(secretidaccessors, indent=self.indent, sort_keys=self.sort_keys)


class SecretIdAccessorListTableFormatter(SecretIdAccessorListFormatter):
    def format_secretidaccessors(self, secretidaccessors: list[dict[str, Any]]) -> str:
        table = Table()
        table.add_column("Secret Accessor ID")
        table.add_column("Creation Time")
        table.add_column("Expiration Time")
        table.add_column("Metadata")

        for secretidaccessor in secretidaccessors:
            table.add_row(
                secretidaccessor["accessor_id"],
                secretidaccessor["creation_time"],
                secretidaccessor["expiration_time"],
                json.dumps(secretidaccessor["metadata"]),
            )

        buffer = StringIO()
        Console(file=buffer).print(table)
        return buffer.getvalue().rstrip()


class SecretIdAccessorListFormat(str, Enum):
    table = "table"
    json = "json"

    @property
    def formatter(self) -> SecretIdAccessorListFormatter:
        if self == SecretIdAccessorListFormat.table:
            return SecretIdAccessorListTableFormatter()
        elif self == SecretIdAccessorListFormat.json:
            return SecretIdAccessorListJsonFormatter()
        else:
            raise ValueError(f"Unknown formatter: {self}")



from __future__ import annotations

from pathlib import Path
from sys import stderr
from typing import Optional

from hvac import Client  # type: ignore[import]
from rich import print
from typer import Option
from typer_builder import Dependencies

from vau.config import Config, VaultAlias

DEFAULT_CONFIG_FILE = Path.home() / ".config" / "vau" / "config.json"


def callback(
    use: str | None = Option(None, help="The vault alias to use", metavar="ALIAS"),
    vault_addr: str | None = Option(None, help="The address of the vault server", envvar="VAULT_ADDR", metavar="URL"),
    token: str | None = Option(None, help="The vault token", envvar="VAULT_TOKEN", metavar="TOKEN"),
    namespace: Optional[str] = Option(
        None,
        help="The vault namespace",
        envvar="VAULT_NAMESPACE",
    ),
    config_file: Path = Option(DEFAULT_CONFIG_FILE, help="The config file", envvar="VAU_CONFIG_FILE"),
    dependencies: Dependencies = Dependencies.Provides(Client, Config, VaultAlias),
) -> None:
    if (use is not None) and (vault_addr is not None):
        print("[red]Cannot specify both --use and --vault-addr[/red]", file=stderr)
        exit(1)

    def get_config() -> Config:
        return Config.from_file(config_file)

    def get_vault_alias() -> VaultAlias:
        nonlocal use
        if use is not None or vault_addr is None:
            config = get_config()
            use = use or config.default_alias
            if not use:
                print("[red]No default vault alias specified[/red]", file=stderr)
                exit(1)
            if use not in config.aliases:
                print(f"[red]Unknown vault alias '{use}'[/red]", file=stderr)
                exit(1)
            return config.aliases[use]
        else:
            return VaultAlias(addr=vault_addr, token=token, namespace=namespace)

    def get_client() -> Client:
        alias = get_vault_alias()
        client = Client(url=alias.addr, token=token or alias.token, namespace=namespace or alias.namespace)
        if not client.is_authenticated():
            print("[red]Failed to authenticate with vault[/red]", file=stderr)
            exit(1)
        return client

    dependencies.set_supplier(Config, get_config)
    dependencies.set_supplier(VaultAlias, get_vault_alias)
    dependencies.set_supplier(Client, get_client)



from __future__ import annotations

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Context, Option


def main(
    client: AppRole,
    ctx: Context,
    secret_id: str | None = Option(None, help="The secret ID to remove", metavar="SECRET_ID"),
    accessor_id: str | None = Option(None, help="The accessor ID of the secret ID to remove", metavar="ACCESSOR_ID"),
) -> None:
    """
    Remove a secret ID from an AppRole by the secret ID or the accessor ID.

    Note that an attempt to remove a secret ID that does not exist will not result in an error.
    """

    role_name = ctx.ensure_object(str)

    if secret_id is not None and accessor_id is not None:
        print("[bold red]Cannot specify both --secret-id and --accessor-id[/bold red]")
        exit(1)

    if secret_id is not None:
        client.destroy_secret_id(role_name, secret_id)
        print(f"Removed secret ID [bold]{secret_id}[/bold] of AppRole [bold]{role_name}[/bold].")
    elif accessor_id is not None:
        client.destroy_secret_id_accessor(role_name, accessor_id)
        print(f"Removed secret ID accessed by [bold]{accessor_id}[/bold] of AppRole [bold]{role_name}[/bold].")
    else:
        print("[bold red]Must specify either --secret-id or --accessor-id[/bold red]")
        exit(1)



import json

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Option


def main(
    client: AppRole,
    role_id: str = Option(..., help="Role ID", metavar="ROLE_ID"),
    secret_id: str = Option(..., help="Secret ID", metavar="SECRET_ID"),
) -> None:
    """
    Login to vault using an Approle and retrieve a token.

    The token and its metadata will be printed to stdout in JSON format. The JSON will look something like this:

    {
        "client_token": "hvs...",
        "accessor": "ecB...",
        "policies": ["default"],
        "token_policies": ["default"],
        "metadata": { "role_name": "test" },
        "lease_duration": 86400,
        "renewable": true,
        "entity_id": "4e939730-4cb2-e996-4c49-ff3914d9749b",
        "token_type": "service",
        "orphan": true,
        "mfa_requirement": null,
        "num_uses": 0
    }
    """

    response = client.login(role_id=role_id, secret_id=secret_id)["auth"]
    print(json.dumps(response, indent=2))



from typer_builder import build_app_from_module


def main() -> None:
    app = build_app_from_module(
        "vau.commands", typer_options={"no_args_is_help": True, "pretty_exceptions_enable": False}
    )
    app()


if __name__ == "__main__":
    main()



from __future__ import annotations

from enum import Enum
from typing import Optional

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Option

from ._format import ApproleFormat
from ._utils import comma_flat_split


class TokenType(str, Enum):
    """The type of token."""

    service = "service"
    batch = "batch"
    default = "default"
    default_service = "default-service"
    default_batch = "default-batch"


def main(
    client: AppRole,
    role_name: str = Option(..., help="The name of the role to create", metavar="ROLE_NAME"),
    role_id: Optional[str] = Option(
        None, help="A custom role ID to use. If not specified, one will be generated.", metavar="ROLE_ID"
    ),
    token_ttl: str = Option("1h", help="The TTL for the token", metavar="DURATION"),
    token_max_ttl: str = Option("24h", help="The max TTL for the token", metavar="DURATION"),
    policies: list[str] | None = Option(None, help="The policies for the token"),
    bind_secret_id: bool = Option(True, help="Whether to bind the secret ID"),
    secret_id_bound_cidrs: list[str] | None = Option(None, help="The CIDRs to bind the secret ID to", metavar="CIDR"),
    secret_id_num_uses: int = Option(0, help="The number of uses for the secret ID"),
    secret_id_ttl: str = Option("1h", help="The TTL for the secret ID", metavar="DURATION"),
    local_secret_ids: bool = Option(False, help="Whether to generate local secret IDs"),
    token_bound_cidrs: list[str] | None = Option(None, help="The CIDRs to bind the token to", metavar="CIDR"),
    token_explicit_max_ttl: str = Option("24h", help="The explicit max TTL for the token", metavar="DURATION"),
    token_no_default_policy: bool = Option(False, help='Whether the "default" policy will be added to the token'),
    token_num_uses: int = Option(0, help="The maximum number of times a token can be used"),
    token_period: str = Option(
        "24h", help="The maximum allowed period value when a periodic token is requested", metavar="DURATION"
    ),
    token_type: TokenType = Option(TokenType.default, help="The type of token"),
    format: ApproleFormat = Option(ApproleFormat.json, "-o", "--format", help="Output format"),
) -> None:
    """Create or update a vault approle."""

    client.create_or_update_approle(
        role_name=role_name,
        bind_secret_id=bind_secret_id,
        secret_id_bound_cidrs=comma_flat_split(secret_id_bound_cidrs or []),
        secret_id_num_uses=secret_id_num_uses,
        secret_id_ttl=secret_id_ttl,
        enable_local_secret_ids=local_secret_ids,
        token_ttl=token_ttl,
        token_max_ttl=token_max_ttl,
        token_policies=comma_flat_split(policies or []),
        token_bound_cidrs=comma_flat_split(token_bound_cidrs or []),
        token_explicit_max_ttl=token_explicit_max_ttl,
        token_no_default_policy=token_no_default_policy,
        token_num_uses=token_num_uses,
        token_period=token_period,
        token_type=token_type,
    )

    if role_id is not None:
        client.update_role_id(role_name, role_id)

    approle = client.read_role(role_name)["data"]
    print(format.formatter.format_approle(approle))



from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from rich import print
from typer import Context, Option


def main(
    client: AppRole,
    ctx: Context,
    role_id: str = Option(..., help="The new role ID for the role", metavar="ROLE_ID"),
) -> None:
    """Update the role ID of an approle."""

    role_name = ctx.ensure_object(str)
    client.update_role_id(role_name, role_id)
    print(f"Updated role ID for role [bold blue]{role_name}[/bold blue] to '{role_id}'.")



from __future__ import annotations

from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from typer import Option

from . import list_roles
from ._format import ApproleListFormat


def main(
    client: AppRole,
    format: ApproleListFormat = Option(ApproleListFormat.table, "-o", "--format", help="Output format"),
) -> None:
    """List vault app roles."""

    roles = list_roles(client)
    print(format.formatter.format_approles(roles))



from hvac.api.auth_methods.approle import AppRole  # type: ignore[import]
from typer import Context, Option

from .._format import SecretIdAccessorListFormat


def main(
    client: AppRole,
    ctx: Context,
    format: SecretIdAccessorListFormat = Option(
        SecretIdAccessorListFormat.table, "-o", "--format", help="Output format"
    ),
) -> None:
    """List the secret id accessors for a vault approle."""

    role_name = ctx.ensure_object(str)
    accessors = [
        {"accessor_id": accessor_id, **client.read_secret_id_accessor(role_name, accessor_id)["data"]}
        for accessor_id in client.list_secret_id_accessors(role_name)["data"]["keys"]
    ]
    print(format.formatter.format_secretidaccessors(accessors))
